{# -*- coding: utf-8 -*-

  This file is part of Invenio.
  Copyright (C) 2015-2020 CERN.

  Invenio is free software; you can redistribute it and/or modify it
  under the terms of the MIT License; see LICENSE file for more details.
#}
{%- extends config.OAUTHCLIENT_LOGIN_USER_TEMPLATE_PARENT %}

{%- from "invenio_oauthclient/_macros.html" import oauth_button %}


{%- block form_outer %}
  {% if config.OAUTHCLIENT_REMOTE_APPS %}
  <div class="ui form">
    {% for name in config.OAUTHCLIENT_REMOTE_APPS.keys() %}
      {{ oauth_button(name, next=request.args.get('next')) }}
    {% endfor %}
  </div>

    <div class="ui horizontal divider">
      Or
    </div>
  {% endif %}
  {{ super () }}
{% endblock %}
