{# -*- coding: utf-8 -*-

  This file is part of Invenio.
  Copyright (C) 2015-2020 CERN.

  Invenio is free software; you can redistribute it and/or modify it
  under the terms of the MIT License; see LICENSE file for more details.
#}

{%- extends config.OAUTHCLIENT_SETTINGS_TEMPLATE %}

{% from "invenio_oauthclient/_macros.html" import render_field %}

{% block page_body %}
<div class="ui centered grid">
  <div class="row">
  {%- block signup_panel %}
  <div>
    {%- block signup_header %}
      <div class="ui center aligned header">
        <div class="content">
          {%- block signup_app_icon %}
          {% if app_icon -%}
          <i class="{{app_icon}}"></i>
          {% endif %}
          {%- endblock %}
          {{ _('Sign-up with %(title)s!', title=app_title) }}
          <div class="sub header">
            {{ _('Fill in your details to complete your registration. You only have to do this once.') }}
          </div>
        </div>
        <hr />
    </div>
    {% endblock %}
    {%- block signup_form %}
    <form class="ui form" name="edit" method="post" role="form">
      {%- for field in form %}
        {{ render_field(field) }}
      {%- endfor %}
    </form>
    {%- endblock %}
  </div>
  {%- endblock %}
</div>
</div>
{% endblock %}
