{# -*- coding: utf-8 -*-

  This file is part of Invenio.
  Copyright (C) 2015-2018 CERN.

  Invenio is free software; you can redistribute it and/or modify it
  under the terms of the MIT License; see LICENSE file for more details.
#}

{% macro render_field(field, icon="", placeholder='', autofocus=False) %}
  <div class="form-group {% if icon %} has-feedback{% endif %}{% if field.errors %} has-error{% endif %}">
    {%- set extras = dict(autofocus="") if autofocus else dict() %}
    {{field(class_="form-control", placeholder=placeholder or _(field.label.text | string), **extras)}}
    {%- if icon %}
    <i class="{{icon}} form-control-feedback" aria-hidden="true" ></i>
    {%- endif%}
  </div>
{% endmacro %}


{% macro form_errors(form) %}
  {%- if form.errors %}
  <div class="alert alert-danger alert-dismissible text-left" role="alert">
    <button type="button" class="close" data-dismiss="alert" aria-label="{{ _('Close') }}">
      <span aria-hidden="true">&times;</span>
    </button>
    {%- for field, errors in form.errors.items() %}
      {%- for error in errors %}
        <p>{{error}}</p>
      {%- endfor %}
    {%- endfor %}
  </div>
  {%- endif %}
{% endmacro %}


{% macro oauth_button(name, next=None) %}
  <a class="btn btn-default btn-lg btn-block"
     href="{{url_for('invenio_oauthclient.login', remote_app=name, next=next or request.referrer)}}">
    <i class="fa fa-{{ name|lower() }}"></i> {{
      _('Sign in with %(title)s', title=config.OAUTHCLIENT_REMOTE_APPS[name]['title'])
    }}
  </a>
{% endmacro %}
