{# -*- coding: utf-8 -*-

  This file is part of Invenio.
  Copyright (C) 2015-2020 CERN.

  Invenio is free software; you can redistribute it and/or modify it
  under the terms of the MIT License; see LICENSE file for more details.
#}

{% macro render_field(field, icon="", placeholder='', autofocus=False) %}
  <div class="field {% if field.errors %} error{% endif %}">
    {%- set extras = dict(autofocus="") if autofocus else dict() %}
    {{field(class_="form-control", placeholder=placeholder or _(field.label.text | string), **extras)}}
    {%- if icon %}
    <i class="{{icon}}" aria-hidden="true" ></i>
    {%- endif%}
  </div>
  {%- if field.errors %}
      <div class="ui pointing red basic label">
        {%- for error in field.errors %}
        <p>{{error}}</p>
        {%- endfor %}
      </div>
    {%- endif %}
{% endmacro %}


{% macro form_errors(form) %}
  {%- if form.errors %}
  <div class="ui negative message" role="alert">
    {%- for field, errors in form.errors.items() %}
      {%- for error in errors %}
        <p>{{error}}</p>
      {%- endfor %}
    {%- endfor %}
  </div>
  {%- endif %}
{% endmacro %}


{% macro oauth_button(name, next=None) %}
<div class="field">
  <a class="fluid ui basic button large"
     href="{{url_for('invenio_oauthclient.login', remote_app=name, next=next or request.referrer)}}">
    <i class="{{ name|lower() }} icon"></i> {{
      _('Sign in with %(title)s', title=config.OAUTHCLIENT_REMOTE_APPS[name]['title'])
    }}
  </a>
</div>
{% endmacro %}
